% New Keynesian 3-equation Model
% code by Yizhou(Kyle) Kuang


%----------------------------------------------------------------
% 0. Housekeeping (close all graphic windows)
%----------------------------------------------------------------

close all;

%----------------------------------------------------------------
% 1. Defining variables
%----------------------------------------------------------------

var  
 r  ${r}$    (long_name='interest rate')
 y  ${y}$    (long_name='output-gap')
 pi  ${\pi}$  (long_name='inflation')
;  


varexo
  e_R  ${\varepsilon^R}$  (long_name='monetary policy shock')
  e_D  ${\varepsilon^D}$  (long_name='demand shock')
  e_S  ${\varepsilon^S}$  (long_name='supply shock')
;
parameters  beta sigma phi theta rho phi_pi phi_y;

%----------------------------------------------------------------
% 2. Calibration
%----------------------------------------------------------------

beta    	= 0.99;				% discount factor
sigma		= 1;				% risk aversion consumption
phi			= 1;				% labor disutility
theta		= 3/4;				% new keynesian Philips Curve, forward term
rho			= 0.5;				% MPR Smoothing
phi_pi		= 1.5;				% MPR Inflation
phi_y		= 0.5/4;			% MPR GDP




%----------------------------------------------------------------
% 3. Model
%----------------------------------------------------------------

model(linear); 
	% IS curve
	y = y(+1) - 1/sigma*(r-pi(+1)) + e_D;
	% AS curve
	pi = beta*pi(+1) + ((1-theta)*(1-beta*theta)/theta)*(sigma+phi)*y + e_S;
	% Monetary Policy Rule
	r = rho*r(-1) +  (1-rho)*( phi_pi*pi+ phi_y*y) + e_R ;
	
end;


%----------------------------------------------------------------
% 4. Simulation
%----------------------------------------------------------------
shocks;
var e_R; stderr 1;
var e_D; stderr 1;
var e_S; stderr 1;
end;

stoch_simul(order=1,periods=100,simul_replic=1,drop=0);
datatomfile('three_eqnk_data_sim',{'y';'r';'pi'})


%----------------------------------------------------------------
% 5. Estimation
%----------------------------------------------------------------

varobs r y pi;

estimated_params;
%	PARAM NAME,	    PRIOR_SHAPE,		PRIOR_mean,		PRIOR_std_err,	PRIOR_P3,		PRIOR_P4,		JSCALE
    beta,               beta_pdf,             0.99,            0.001,           0.9,              0.999               ;
    sigma,              gamma_pdf,                1,            0.5                                                   ;
    phi  ,              gamma_pdf,              0.8,            0.5                                                   ; 
    theta,            normal_pdf,               0.6,            1                                                     ;     
    phi_pi,           normal_pdf,               1.5,           1                                                      ;
    phi_y ,           normal_pdf,               0.5,           1                                                      ;
end;

steady;
check;
model_diagnostics;
identification;

estimation(order=1, datafile='three_eqnk_data_sim.m',presample=0,first_obs=1,mode_check
,prefilter=0,plot_priors=0,lik_init=1,mh_nblocks=1
,mh_jscale=0.85,mh_drop=0.5
,mode_compute=5,prior_trunc=0,silent_optimizer
,mh_replic=1000
,mh_conf_sig=0.90,conf_sig=0.90) r y pi;


posterior_function(function = get_policy_rules, sampling_draws = 50);
oo_.posterior_function_results